# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.list_saml_connections_response import ListSamlConnectionsResponse
from ..core.pydantic_utilities import parse_obj_as
from ..errors.bad_request_error import BadRequestError
from ..types.api_error import ApiError as types_api_error_ApiError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError as core_api_error_ApiError
import datetime as dt
from ..types.create_saml_connection_response import CreateSamlConnectionResponse
from ..types.get_saml_connection_response import GetSamlConnectionResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.delete_saml_connection_response import DeleteSamlConnectionResponse
from ..types.update_saml_connection_response import UpdateSamlConnectionResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SamlConnectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_saml_connections(
        self,
        *,
        organization_id: typing.Optional[str] = None,
        page_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSamlConnectionsResponse:
        """
        List SAML Connections.

        Parameters
        ----------
        organization_id : typing.Optional[str]
            The Organization ID.

        page_token : typing.Optional[str]
            A pagination token. Leave empty to get the first page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSamlConnectionsResponse
            OK

        Examples
        --------
        from tesseral import Tesseral

        client = Tesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )
        client.saml_connections.list_saml_connections()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/saml-connections",
            method="GET",
            params={
                "organizationId": organization_id,
                "pageToken": page_token,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListSamlConnectionsResponse,
                    parse_obj_as(
                        type_=ListSamlConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    def create_saml_connection(
        self,
        *,
        id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        create_time: typing.Optional[dt.datetime] = OMIT,
        update_time: typing.Optional[dt.datetime] = OMIT,
        primary: typing.Optional[bool] = OMIT,
        sp_acs_url: typing.Optional[str] = OMIT,
        sp_entity_id: typing.Optional[str] = OMIT,
        idp_redirect_url: typing.Optional[str] = OMIT,
        idp_x509certificate: typing.Optional[str] = OMIT,
        idp_entity_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSamlConnectionResponse:
        """
        Create a SAML Connection.

        Parameters
        ----------
        id : typing.Optional[str]
            The SAML Connection ID. Starts with `saml_connection_...`.

        organization_id : typing.Optional[str]
            The Organization this SAML Connection belongs to.

        create_time : typing.Optional[dt.datetime]
            When the SAML Connection was created.

        update_time : typing.Optional[dt.datetime]
            When the SAML Connection was last updated.

        primary : typing.Optional[bool]
            Whether this is the primary SAML Connection for the Organization.

        sp_acs_url : typing.Optional[str]
            The Service Provider Assertion Consumer Service (ACS) URL.

        sp_entity_id : typing.Optional[str]
            The Service Provider Entity ID.

        idp_redirect_url : typing.Optional[str]
            The Identity Provider Redirect URL.

        idp_x509certificate : typing.Optional[str]
            The Identity Provider certificate, in PEM-encoded X.509 format.

             Starts with `----BEGIN CERTIFICATE----`.

        idp_entity_id : typing.Optional[str]
            The Identity Provider Entity ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSamlConnectionResponse
            OK

        Examples
        --------
        from tesseral import Tesseral

        client = Tesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )
        client.saml_connections.create_saml_connection()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/saml-connections",
            method="POST",
            json={
                "id": id,
                "organizationId": organization_id,
                "createTime": create_time,
                "updateTime": update_time,
                "primary": primary,
                "spAcsUrl": sp_acs_url,
                "spEntityId": sp_entity_id,
                "idpRedirectUrl": idp_redirect_url,
                "idpX509Certificate": idp_x509certificate,
                "idpEntityId": idp_entity_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSamlConnectionResponse,
                    parse_obj_as(
                        type_=CreateSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    def get_saml_connection(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSamlConnectionResponse:
        """
        Get a SAML Connection.

        Parameters
        ----------
        id : str
            The SAML Connection ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSamlConnectionResponse
            OK

        Examples
        --------
        from tesseral import Tesseral

        client = Tesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )
        client.saml_connections.get_saml_connection(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetSamlConnectionResponse,
                    parse_obj_as(
                        type_=GetSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    def delete_saml_connection(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteSamlConnectionResponse:
        """
        Delete a SAML Connection.

        Parameters
        ----------
        id : str
            The SAML Connection ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteSamlConnectionResponse
            OK

        Examples
        --------
        from tesseral import Tesseral

        client = Tesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )
        client.saml_connections.delete_saml_connection(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    DeleteSamlConnectionResponse,
                    parse_obj_as(
                        type_=DeleteSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    def update_saml_connection(
        self,
        id_: str,
        *,
        id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        create_time: typing.Optional[dt.datetime] = OMIT,
        update_time: typing.Optional[dt.datetime] = OMIT,
        primary: typing.Optional[bool] = OMIT,
        sp_acs_url: typing.Optional[str] = OMIT,
        sp_entity_id: typing.Optional[str] = OMIT,
        idp_redirect_url: typing.Optional[str] = OMIT,
        idp_x509certificate: typing.Optional[str] = OMIT,
        idp_entity_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSamlConnectionResponse:
        """
        Update a SAML Connection.

        Parameters
        ----------
        id_ : str
            The SAML Connection ID.

        id : typing.Optional[str]
            The SAML Connection ID. Starts with `saml_connection_...`.

        organization_id : typing.Optional[str]
            The Organization this SAML Connection belongs to.

        create_time : typing.Optional[dt.datetime]
            When the SAML Connection was created.

        update_time : typing.Optional[dt.datetime]
            When the SAML Connection was last updated.

        primary : typing.Optional[bool]
            Whether this is the primary SAML Connection for the Organization.

        sp_acs_url : typing.Optional[str]
            The Service Provider Assertion Consumer Service (ACS) URL.

        sp_entity_id : typing.Optional[str]
            The Service Provider Entity ID.

        idp_redirect_url : typing.Optional[str]
            The Identity Provider Redirect URL.

        idp_x509certificate : typing.Optional[str]
            The Identity Provider certificate, in PEM-encoded X.509 format.

             Starts with `----BEGIN CERTIFICATE----`.

        idp_entity_id : typing.Optional[str]
            The Identity Provider Entity ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSamlConnectionResponse
            OK

        Examples
        --------
        from tesseral import Tesseral

        client = Tesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )
        client.saml_connections.update_saml_connection(
            id_="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id_)}",
            method="PATCH",
            json={
                "id": id,
                "organizationId": organization_id,
                "createTime": create_time,
                "updateTime": update_time,
                "primary": primary,
                "spAcsUrl": sp_acs_url,
                "spEntityId": sp_entity_id,
                "idpRedirectUrl": idp_redirect_url,
                "idpX509Certificate": idp_x509certificate,
                "idpEntityId": idp_entity_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateSamlConnectionResponse,
                    parse_obj_as(
                        type_=UpdateSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSamlConnectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_saml_connections(
        self,
        *,
        organization_id: typing.Optional[str] = None,
        page_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSamlConnectionsResponse:
        """
        List SAML Connections.

        Parameters
        ----------
        organization_id : typing.Optional[str]
            The Organization ID.

        page_token : typing.Optional[str]
            A pagination token. Leave empty to get the first page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSamlConnectionsResponse
            OK

        Examples
        --------
        import asyncio

        from tesseral import AsyncTesseral

        client = AsyncTesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )


        async def main() -> None:
            await client.saml_connections.list_saml_connections()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/saml-connections",
            method="GET",
            params={
                "organizationId": organization_id,
                "pageToken": page_token,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListSamlConnectionsResponse,
                    parse_obj_as(
                        type_=ListSamlConnectionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    async def create_saml_connection(
        self,
        *,
        id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        create_time: typing.Optional[dt.datetime] = OMIT,
        update_time: typing.Optional[dt.datetime] = OMIT,
        primary: typing.Optional[bool] = OMIT,
        sp_acs_url: typing.Optional[str] = OMIT,
        sp_entity_id: typing.Optional[str] = OMIT,
        idp_redirect_url: typing.Optional[str] = OMIT,
        idp_x509certificate: typing.Optional[str] = OMIT,
        idp_entity_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateSamlConnectionResponse:
        """
        Create a SAML Connection.

        Parameters
        ----------
        id : typing.Optional[str]
            The SAML Connection ID. Starts with `saml_connection_...`.

        organization_id : typing.Optional[str]
            The Organization this SAML Connection belongs to.

        create_time : typing.Optional[dt.datetime]
            When the SAML Connection was created.

        update_time : typing.Optional[dt.datetime]
            When the SAML Connection was last updated.

        primary : typing.Optional[bool]
            Whether this is the primary SAML Connection for the Organization.

        sp_acs_url : typing.Optional[str]
            The Service Provider Assertion Consumer Service (ACS) URL.

        sp_entity_id : typing.Optional[str]
            The Service Provider Entity ID.

        idp_redirect_url : typing.Optional[str]
            The Identity Provider Redirect URL.

        idp_x509certificate : typing.Optional[str]
            The Identity Provider certificate, in PEM-encoded X.509 format.

             Starts with `----BEGIN CERTIFICATE----`.

        idp_entity_id : typing.Optional[str]
            The Identity Provider Entity ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateSamlConnectionResponse
            OK

        Examples
        --------
        import asyncio

        from tesseral import AsyncTesseral

        client = AsyncTesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )


        async def main() -> None:
            await client.saml_connections.create_saml_connection()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/saml-connections",
            method="POST",
            json={
                "id": id,
                "organizationId": organization_id,
                "createTime": create_time,
                "updateTime": update_time,
                "primary": primary,
                "spAcsUrl": sp_acs_url,
                "spEntityId": sp_entity_id,
                "idpRedirectUrl": idp_redirect_url,
                "idpX509Certificate": idp_x509certificate,
                "idpEntityId": idp_entity_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateSamlConnectionResponse,
                    parse_obj_as(
                        type_=CreateSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    async def get_saml_connection(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSamlConnectionResponse:
        """
        Get a SAML Connection.

        Parameters
        ----------
        id : str
            The SAML Connection ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSamlConnectionResponse
            OK

        Examples
        --------
        import asyncio

        from tesseral import AsyncTesseral

        client = AsyncTesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )


        async def main() -> None:
            await client.saml_connections.get_saml_connection(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetSamlConnectionResponse,
                    parse_obj_as(
                        type_=GetSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_saml_connection(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteSamlConnectionResponse:
        """
        Delete a SAML Connection.

        Parameters
        ----------
        id : str
            The SAML Connection ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteSamlConnectionResponse
            OK

        Examples
        --------
        import asyncio

        from tesseral import AsyncTesseral

        client = AsyncTesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )


        async def main() -> None:
            await client.saml_connections.delete_saml_connection(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    DeleteSamlConnectionResponse,
                    parse_obj_as(
                        type_=DeleteSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)

    async def update_saml_connection(
        self,
        id_: str,
        *,
        id: typing.Optional[str] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        create_time: typing.Optional[dt.datetime] = OMIT,
        update_time: typing.Optional[dt.datetime] = OMIT,
        primary: typing.Optional[bool] = OMIT,
        sp_acs_url: typing.Optional[str] = OMIT,
        sp_entity_id: typing.Optional[str] = OMIT,
        idp_redirect_url: typing.Optional[str] = OMIT,
        idp_x509certificate: typing.Optional[str] = OMIT,
        idp_entity_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSamlConnectionResponse:
        """
        Update a SAML Connection.

        Parameters
        ----------
        id_ : str
            The SAML Connection ID.

        id : typing.Optional[str]
            The SAML Connection ID. Starts with `saml_connection_...`.

        organization_id : typing.Optional[str]
            The Organization this SAML Connection belongs to.

        create_time : typing.Optional[dt.datetime]
            When the SAML Connection was created.

        update_time : typing.Optional[dt.datetime]
            When the SAML Connection was last updated.

        primary : typing.Optional[bool]
            Whether this is the primary SAML Connection for the Organization.

        sp_acs_url : typing.Optional[str]
            The Service Provider Assertion Consumer Service (ACS) URL.

        sp_entity_id : typing.Optional[str]
            The Service Provider Entity ID.

        idp_redirect_url : typing.Optional[str]
            The Identity Provider Redirect URL.

        idp_x509certificate : typing.Optional[str]
            The Identity Provider certificate, in PEM-encoded X.509 format.

             Starts with `----BEGIN CERTIFICATE----`.

        idp_entity_id : typing.Optional[str]
            The Identity Provider Entity ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSamlConnectionResponse
            OK

        Examples
        --------
        import asyncio

        from tesseral import AsyncTesseral

        client = AsyncTesseral(
            backend_api_key="YOUR_BACKEND_API_KEY",
        )


        async def main() -> None:
            await client.saml_connections.update_saml_connection(
                id_="id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/saml-connections/{jsonable_encoder(id_)}",
            method="PATCH",
            json={
                "id": id,
                "organizationId": organization_id,
                "createTime": create_time,
                "updateTime": update_time,
                "primary": primary,
                "spAcsUrl": sp_acs_url,
                "spEntityId": sp_entity_id,
                "idpRedirectUrl": idp_redirect_url,
                "idpX509Certificate": idp_x509certificate,
                "idpEntityId": idp_entity_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateSamlConnectionResponse,
                    parse_obj_as(
                        type_=UpdateSamlConnectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        types_api_error_ApiError,
                        parse_obj_as(
                            type_=types_api_error_ApiError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise core_api_error_ApiError(status_code=_response.status_code, body=_response.text)
        raise core_api_error_ApiError(status_code=_response.status_code, body=_response_json)
